<?php
/**
 * EPL Functions: Spam Titan integration into mail app
 *
 * @link http://www.egroupware.org
 * @package epl
 * @subpackage mail
 * @author Hadi Nategh <hn@egroupware.org>
 * @copyright (c) 2016 by EGroupware GmbH
 * @version $Id$
 */

use EGroupware\Api;

/**
 * Spam Titan actions integration into mail app
 *
 */
class stylite_mail_spamtitan
{
	/**
	 * stores items to be processed on shutdown callback
	 *
	 * @var array
	 */
	protected static $items = array();

	/**
	 * Spam Titan actions
	 */
	public static function getActions()
	{
		return array (
			'whitelist' => array (
				'caption' => 'Whitelist',
				'icon'	=> 'whitelist',
				'children' => array(
					'whitelist_email_add' => array(
						'caption'	=> 'Add to whitelisted mails',
						'icon'		=> 'spam_list_add',
						'hint'		=> 'Whitelisted mails are not marked as spam',
						'onExecute' => 'javaScript:app.mail.spam_actions',
						'enabled'	=> 'javaScript:app.mail.spamTitan_setActionTitle'
					),
					'whitelist_email_remove' => array(
						'caption'	=> 'Remove from whitelisted mails',
						'icon'		=> 'spam_list_remove',
						'onExecute' => 'javaScript:app.mail.spam_actions',
						'enabled'	=> 'javaScript:app.mail.spamTitan_setActionTitle'
					),
					'whitelist_domain_add' => array(
						'caption'	=> 'Add domain to whitelisted domains',
						'icon'		=> 'spam_list_domain_add',
						'hint'		=> 'Emails in whitelisted domains are not marked as spam',
						'onExecute' => 'javaScript:app.mail.spam_actions',
						'enabled'	=> 'javaScript:app.mail.spamTitan_setActionTitle'
					),
					'whitelist_domain_remove' => array(
						'caption'	=> 'Remove domain from whitelisted domains',
						'icon'		=> 'spam_list_domain_remove',
						'onExecute' => 'javaScript:app.mail.spam_actions',
						'enabled'	=> 'javaScript:app.mail.spamTitan_setActionTitle'
					)
				)
			),
			'blacklist' => array (
				'caption' => 'Blacklist',
				'icon' => 'blacklist',
				'children' => array(
					'blacklist_email_add' => array(
						'caption'	=> 'Add to blacklisted mails',
						'icon'		=> 'spam_list_add',
						'hint'		=> 'Blacklisted mails are marked as spam',
						'onExecute' => 'javaScript:app.mail.spam_actions',
						'enabled'	=> 'javaScript:app.mail.spamTitan_setActionTitle'
					),
					'blacklist_email_remove' => array(
						'caption'	=> 'Remove from blacklisted mails',
						'icon'		=> 'spam_list_remove',
						'onExecute' => 'javaScript:app.mail.spam_actions',
						'enabled'	=> 'javaScript:app.mail.spamTitan_setActionTitle'
					),
					'blacklist_domain_add' => array(
						'caption'	=> 'Add domain to blacklisted domains',
						'icon'		=> 'spam_list_domain_add',
						'hint'		=> 'Emails in blacklisted domains are marked as spam',
						'onExecute' => 'javaScript:app.mail.spam_actions',
						'enabled'	=> 'javaScript:app.mail.spamTitan_setActionTitle'
					),
					'blacklist_domain_remove' => array(
						'caption'	=> 'Remove domain from blacklisted domains',
						'icon'		=> 'spam_list_domain_remove',
						'onExecute' => 'javaScript:app.mail.spam_actions',
						'enabled'	=> 'javaScript:app.mail.spamTitan_setActionTitle'
					)
				)
			)
		);
	}

	/**
	 * Set items to be processed on shutdown call
	 *
	 * @param string $_action action name
	 * @param array $_items an array of params
	 * @param array $_data
	 */
	public static function setActionItems ($_action, $_items, $_data)
	{
		foreach ($_items as &$item)
		{
			self::$items [] = array(
				'params' => $item,
				'action' => $_action,
				'data' => $_data
			);
		}
		Api\Egw::on_shutdown(array(__CLASS__, 'runActionForItems'));
	}

	/**
	 * Callback to run action on items. It gets called on shutdown
	 */
	public static function runActionForItems ()
	{
		$push = new Api\Json\Push($GLOBALS['egw_info']['user']['account_id']);
		while(self::$items)
		{
			$args = array_shift(self::$items);
			$result = self::execAction($args['action'], $args['params'],$args['data']);
			$push->call('egw.message', $result);
		}
	}

	/**
	 * Method to execute actions via spamTitan API
	 * @param string $_action action name
	 * @param array $_params
	 * @param array $_data
	 */
	public static function execAction ($_action, $_params, $_data)
	{
		$spamtitan = new stylite_spamtitan($_data);

		//check if user policy exists otherwise create it for the user.
		if (($exists_res = $spamtitan->policy_exists($_data['user'])) !== true
				&& ($policy_res = $spamtitan->add_policy($_data['user'])) !== true)
		{
			return $exists_res.$policy_res;
		}
		switch ($_action)
		{
			case 'spam':
				return (($resp = $spamtitan->learnmail($_params['mailbody'], 'Spam')) === true) ?
					lang('Message learned as Spam and moved to Spam folder') :
					lang('Learning as Spam failed because of %1', $resp);
			case 'ham':
				return (($resp = $spamtitan->learnmail($_params['mailbody'], 'Ham')) === true) ?
					lang('Message learned as Ham (Not Spam) and moved back to Inbox') :
					lang('Learning as Ham (Not Spam) failed because of %1', $resp);
			case 'whitelist_domain_add':
			case 'whitelist_email_add':
				return (($resp = $spamtitan->add_whitelist_entry($_params['sender'], $_data['user'])) === true) ?
					lang('%1 has been added to whiltelisted email addresses', $_params['sender']) :
					lang('Adding %1 to whiltelisted email addresses failed because of %2', $_params['sender'],$resp[0]);
			case 'blacklist_domain_add':
			case 'blacklist_email_add':
				return (($resp = $spamtitan->add_blacklist_entry($_params['sender'], $_data['user'])) === true) ?
					lang('%1 has been added to blacklisted email addresses', $_params['sender']) :
					lang('Adding %1 to blacklisted email addresses failed because of %2', $_params['sender'],$resp[0]);
			case 'whitelist_domain_remove':
			case 'whitelist_email_remove':
				return (($resp = $spamtitan->delete_whitelist_entry($_params['sender'], $_data['user'])) === true) ?
					lang('%1 has been removed from whiltelisted email addresses', $_params['sender']) :
					lang('Removing %1 from whiltelisted email addresses failed because of %2', $_params['sender'],$resp[0]);
			case 'blacklist_domain_remove':
			case 'blacklist_email_remove':
				return (($resp = $spamtitan->delete_blacklist_entry($_params['sender'], $_data['user'])) === true) ?
					lang('%1 has been removed from blacklisted email addresses', $_params['sender']) :
					lang('Removing %1 from blacklisted email addresses failed because of %2', $_params['sender'],$resp[0]);
		}
	}
}